unit SchrinkPanel;
{
	TSchrinkPanel works similar to the interface mechanism used in 3D Studio Max.
	Based on a TPanel it can be used to manage large amounts of controls in
	collapsable panels. TSchrinkPanel is aware of other TSchrinkPanels in the same
	Parent and will realign other TSchrinkPanels when it collapses or expands
	provided the other TSchrinkPanels have the same Left and Width values.
	TSchrinkPanel is also dragable in vertical direction and will drag other
	TSchrinkPanels along provided the other TSchrinkPanels have the same Left and
	Width values.

	The result is a 3DStudio lookalike collapsable and dragable interface
	that can be created with a set of TSchrinkBoxes without a single line of code!

	By Paul van Dinther  dinther@geocities.com 04/01/2000
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TSchrinkPanel = class(tpanel)
  private
		Fbutton: Tpanel;
		FTop: Integer;
		FBottom: Integer;
		FCollapsable: Boolean;
		FDragable: Boolean;
		FbuttonCaption: String;
		FExpand: Boolean;
		FOldHeight: Integer;
		FDragOldY: Integer;
		procedure SetExpand(Value: Boolean);
		procedure SetbuttonCaption(Value: String);
		procedure ButtonClick(Sender: TObject);
		procedure MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
		procedure MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
	protected
		{ Protected declarations }
	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
	published
		{ Published declarations }
		property ButtonCaption: String read FbuttonCaption write SetButtonCaption;
		property Expand: Boolean read FExpand write SetExpand;
		property collapsable: boolean read FCollapsable write FCollapsable;
		property Dragable: Boolean read FDragable write FDragable;
	end;

procedure Register;

implementation

procedure Register;
begin
	RegisterComponents('DPD', [TSchrinkPanel]);
end;

constructor TSchrinkPanel.Create(AOwner: TComponent);
begin
	inherited create(AOwner);
	width := 169;
	Height := 81;
	FButton := TPanel.Create(Self);
	FButton.Parent := Self;
	Fbutton.Top := 0;
	FButton.Left := 0;
	Fbutton.Align := alTop;
	FButton.Height := 16;
	FButton.Color := cl3DLight;
	Fbutton.BevelOuter := bvLowered;
	ButtonCaption := 'Schrink/Expand';
//	Fbutton.OnClick := ButtonClick;
	FButton.OnMouseDown := MouseDown;
	FExpand := True;
	FoldHeight := Height;
	BevelOuter := bvLowered;
	BevelInner := bvRaised;
	FDragable := True;
	FCollapsable := True;
	Caption := '';
	OnMouseMove := MouseMove;
	OnMouseDown := MouseDown;
end;

destructor TSchrinkPanel.Destroy;
begin
	inherited Destroy;
	//FButton.Free;
end;

procedure TSchrinkPanel.SetButtonCaption(Value: String);
begin
	if Value = FbuttonCaption then exit;
	FbuttonCaption := Value;
	Fbutton.Caption := Value;
	Invalidate;
end;

procedure TSchrinkPanel.ButtonClick(Sender: TObject);
begin
	if FExpand then begin
		if not FCollapsable then exit;
		//do the schrink
		Expand := False;
	end else begin
		//do the Expand
		Expand := True;
	end;
end;

procedure TSchrinkPanel.SetExpand(Value: Boolean);
var
	LVert: Integer;
	i: Integer;
	LSchrinkPanel: TSchrinkPanel;
	LOldHeight: Integer;
begin
	if Value = FExpand then exit;
	FExpand := Value;
	LOldHeight := Height;
	if FExpand then begin
		//Expand
		Fbutton.BevelOuter := bvLowered;
		Fbutton.Color := cl3DLight;
		LVert := Height - FoldHeight;
		SetBounds(Left,Top,Width,FoldHeight);
	end else begin
		//Schrink
		Fbutton.BevelOuter := bvRaised;
		Fbutton.Color := clBtnFace;
		LVert := Height - Fbutton.Height - 3 ;
		FoldHeight := Height;
		SetBounds(Left,Top,width,Fbutton.Height + 3);
	end;
	//Move down all SchrinkPanels that have the same Left and width
	for i := 0 to Parent.ControlCount - 1 do begin
		if parent.Controls[i] is TSchrinkPanel then begin
			LSchrinkPanel := TSchrinkPanel(Parent.controls[i]);
			if LSchrinkPanel <> Self then begin
				if (LSchrinkPanel.Top > Top + LOldHeight) and(LSchrinkPanel.Left = Left) and (LSchrinkPanel.Width = width) then begin
					LSchrinkPanel.Top := LSchrinkPanel.Top - LVert;
				end;
			end;
		end;
	end;
end;

procedure TSchrinkPanel.MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
	LSchrinkPanel: TSchrinkPanel;
	LOtherSchrinkPanel: TSchrinkPanel;
	i: Integer;
	Lchange: Integer;
	LDragAllowed: Boolean;
begin
	if (ssLeft in shift) and FDragable then begin
		if sender is TSchrinkPanel then begin
			LSchrinkPanel := TSchrinkPanel(sender);
			Lchange := FDragOldY - Y;
			LDragAllowed := False;
			if LChange < 0 then begin
				//Move Down
				LDragAllowed := (FTop < 0);
				//Adjust LChange if required
				if LChange < FTop then LChange := FTop;
			end else begin
				//Move Up
				LDragAllowed := (Fbottom > Parent.Height);
				//adjust LChange if required
				if LChange > FBottom - Parent.Height then LChange := FBottom - Parent.Height;
			end;
			if LDragAllowed then begin

				FBottom := FBottom - LChange;
				FTop := FTop - LChange;

			for i := 0 to LSchrinkPanel.Parent.controlcount - 1 do begin

				if LSchrinkPanel.Parent.Controls[i] is TSchrinkPanel then begin
					LOtherSchrinkPanel := TSchrinkPanel(LSchrinkPanel.Parent.Controls[i]);
					if LOtherSchrinkPanel.Dragable then begin
						if (LOtherSchrinkPanel.Left = LSchrinkPanel.Left) and (LOtherSchrinkPanel.Width = LSchrinkPanel.Width) then begin
							LOtherSchrinkPanel.Top := LOtherSchrinkPanel.Top - Lchange;
						end;
					end;
				end;
			end;

			end;

		end;
	end;
end;

procedure TSchrinkPanel.MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
	LSchrinkPanel: TSchrinkPanel;
	i: Integer;
begin
	if Y < Fbutton.Height then begin
		if FExpand then begin
			if not FCollapsable then exit;
			//do the schrink
			Expand := False;
		end else begin
			//do the Expand
			Expand := True;
		end;
	end else begin
		FDragOldY := y;
		FTop := Top;
		FBottom := Top + Height;
		for i := 0 to Parent.ControlCount - 1 do begin
			if parent.controls[i] is TSchrinkPanel then begin
				LSchrinkPanel := TSchrinkPanel(parent.controls[i]);
				if (LSchrinkPanel.Left = Left)and (LSchrinkPanel.Width = Width) then begin
					//find top and bottom of panel stack
					if LSchrinkPanel.Top < FTop then FTop := LSchrinkPanel.Top;
					if LschrinkPanel.Top + LSchrinkPanel.Height > FBottom then FBottom := LschrinkPanel.Top + LSchrinkPanel.Height;
				end;
			end;
		end;
	end;
end;

end.
